'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var module$1 = require('module');
var path = require('path');
var vm = require('vm');
var nanoid = require('nanoid');
var url = require('url');
var esbuild = require('esbuild');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var path__default = /*#__PURE__*/_interopDefaultLegacy(path);
var vm__default = /*#__PURE__*/_interopDefaultLegacy(vm);
var url__default = /*#__PURE__*/_interopDefaultLegacy(url);

const isInESModuleScope = () => {
  try {
    return module === void 0;
  } catch {
    return true;
  }
};
const isVMModuleAvailable = () => vm__default['default'].Module !== void 0;
const fileURLToPath = (value) => value.startsWith("file://") ? url__default['default'].fileURLToPath(new URL(value)) : value;
const FUNCTION_NAMES = [
  "getCallerDirname",
  "requireFromString",
  "importFromString",
  "importFromStringSync",
  "processTicksAndRejections"
];
const getCallerDirname = () => {
  const _prepareStackTrace = Error.prepareStackTrace;
  Error.prepareStackTrace = (_err, stackTraces) => stackTraces;
  const callSites = new Error().stack.filter((callSite) => !FUNCTION_NAMES.includes(String(callSite.getFunctionName())));
  Error.prepareStackTrace = _prepareStackTrace;
  const callerFilename = callSites[0].getFileName();
  return path__default['default'].dirname(callerFilename !== null ? fileURLToPath(callerFilename) : process.argv[1]);
};
const resolveModuleSpecifier = (specifier, dirname) => {
  const specifierPath = fileURLToPath(specifier);
  return new RegExp(`^[.\\${path__default['default'].sep}]`).test(specifierPath) ? path__default['default'].resolve(dirname, specifierPath) : specifier;
};

const requireFromString = (code, { dirname, globals } = {}) => {
  var _a;
  const moduleDirname = dirname != null ? dirname : getCallerDirname();
  const moduleFilename = path__default['default'].join(moduleDirname, `${nanoid.nanoid()}.js`);
  const mainModule = isInESModuleScope() ? void 0 : require.main;
  const contextModule = new module$1.Module(moduleFilename, mainModule);
  contextModule.filename = moduleFilename;
  contextModule.path = moduleDirname;
  contextModule.paths = (_a = mainModule == null ? void 0 : mainModule.paths) != null ? _a : [];
  contextModule.require = module$1.createRequire(moduleFilename);
  vm__default['default'].runInNewContext(code, {
    __dirname: contextModule.path,
    __filename: contextModule.filename,
    exports: contextModule.exports,
    module: contextModule,
    require: contextModule.require,
    ...globals
  }, {
    async importModuleDynamically(specifier) {
      return await import(resolveModuleSpecifier(specifier, contextModule.path));
    }
  });
  contextModule.loaded = true;
  return contextModule.exports;
};

const ERR_REQUIRE_ESM = "ERR_REQUIRE_ESM";
const IMPORT_META_URL_SHIM = "var import_meta_url = require('url').pathToFileURL(__filename).toString();";
const IMPORT_META_RESOLVE_SHIM = `var import_meta_resolve = () => {
  throw new Error(
    \`'import.meta.resolve' is not supported
Use asynchronous function 'importFromString' and enable '--experimental-vm-modules' CLI option.
Or use 'transformOptions' to include a polyfill. See https://github.com/evanw/esbuild/issues/1492#issuecomment-893144483 as an example.\`
  );
};`;
const getCJS = (transformOptions) => {
  return {
    ...transformOptions,
    banner: `${(transformOptions == null ? void 0 : transformOptions.banner) !== void 0 ? `${transformOptions.banner}
` : ""}${IMPORT_META_URL_SHIM}
${IMPORT_META_RESOLVE_SHIM}`,
    define: {
      "import.meta.url": "import_meta_url",
      "import.meta.resolve": "import_meta_resolve",
      ...transformOptions == null ? void 0 : transformOptions.define
    },
    format: "cjs"
  };
};
const importFromString = async (code, { dirname, globals, transformOptions } = {}) => {
  if (!isVMModuleAvailable()) {
    const { code: transformedCode2 } = await esbuild.transform(code, getCJS(transformOptions));
    try {
      return requireFromString(transformedCode2, { dirname, globals });
    } catch (err) {
      if (err.code === ERR_REQUIRE_ESM) {
        throw new Error(`'import' statement of ES modules is not supported
Enable '--experimental-vm-modules' CLI option or replace it with dynamic 'import()' expression.`);
      }
      throw err;
    }
  }
  const moduleDirname = dirname != null ? dirname : getCallerDirname();
  const moduleFilename = path__default['default'].join(moduleDirname, `${nanoid.nanoid()}.js`);
  const context = vm__default['default'].createContext({
    __imports__: {},
    ...globals
  });
  const { code: transformedCode = void 0 } = transformOptions !== void 0 ? await esbuild.transform(code, {
    format: "esm",
    ...transformOptions
  }) : {};
  const vmModule = new vm__default['default'].SourceTextModule(transformedCode != null ? transformedCode : code, {
    identifier: moduleFilename,
    context,
    initializeImportMeta(meta) {
      meta.url = moduleFilename;
    },
    async importModuleDynamically(specifier) {
      return await import(resolveModuleSpecifier(specifier, moduleDirname));
    }
  });
  const linker = async (specifier) => {
    const resolvedSpecifier = resolveModuleSpecifier(specifier, moduleDirname);
    const targetModule = await import(resolvedSpecifier);
    context.__imports__[specifier] = targetModule;
    const exportedNames = new Set(Object.getOwnPropertyNames(targetModule));
    const targetModuleContent = `${exportedNames.delete("default") ? `export default __imports__['${specifier}'].default;
` : ""}export const { ${[...exportedNames].join(", ")} } = __imports__['${specifier}'];`;
    return new vm__default['default'].SourceTextModule(targetModuleContent, {
      identifier: resolvedSpecifier,
      context
    });
  };
  await vmModule.link(linker);
  await vmModule.evaluate();
  return vmModule.namespace;
};
const importFromStringSync = (code, { dirname, globals, transformOptions } = {}) => {
  const { code: transformedCode } = esbuild.transformSync(code, getCJS(transformOptions));
  try {
    return requireFromString(transformedCode, { dirname, globals });
  } catch (err) {
    if (err.code === ERR_REQUIRE_ESM) {
      throw new Error(`'import' statement of ES modules is not supported
Use asynchronous function 'importFromString' instead or replace it with dynamic 'import()' expression.`);
    }
    throw err;
  }
};

exports.importFromString = importFromString;
exports.importFromStringSync = importFromStringSync;
exports.requireFromString = requireFromString;
